/*************************************************************************
* File Name          : RemoteControlDozer_v2.ino
* Author             : Evan
* Updated            : Evan
* Version            : V0.2.0
* Date               : 5/16/2013
* Description        : Example for Advanced Kit. Use the motor driver on 
                       base shield. Arm motor use the Me-Motor Driver.
* License            : CC-BY-SA 3.0
* Copyright (C) 2011 Hulu Robot Technology Co., Ltd. All right reserved.
* http://www.makeblock.cc/
**************************************************************************/

#include <Me_BaseShield.h>
#include <Me_MotorDriver.h>
#include <SoftwareSerial.h>
#include <Me_InfraredReceiverDecode.h>
#include <Me_BaseShieldMotorDriver.h>

Me_BaseShieldMotorDriver baseShieldMotorDriver;// use M1 and M2 ports on BaseShield
Me_MotorDriver motorDriver(PORT_1); 
Me_InfraredReceiverDecode infraredReceiverDecode(PORT_6);

int leftSpeed = 200;
int rightSpeed = 200;
int turnSpeed = 128;
int armSpeed = 255;

void setup()
{
    // initialize serial communication with computer:
    Serial.begin(9600);
    // initialize BaseShield Motor Driver:
    baseShieldMotorDriver.begin();
    // initialize Motor Driver:
    motorDriver.begin();
    // initialize Infrared Receiver Decode:
    infraredReceiverDecode.begin();
    Serial.println("Start!");
}

void loop()
{
    if(infraredReceiverDecode.available())
    {
        switch(infraredReceiverDecode.read())
        {
            case IR_PLUS_BUTTON:
                  runForward();while(infraredReceiverDecode.buttonState());
                  runStop();break;
            case IR_MINUS_BUTTON:
                  runBack();while(infraredReceiverDecode.buttonState());
                  runStop();break;
            case IR_PREVIOUS_BUTTON:
                  runLeft();while(infraredReceiverDecode.buttonState());
                  runStop();break;
            case IR_NEXT_BUTTON:
                  runRight();while(infraredReceiverDecode.buttonState());
                  runStop();break;
            case IR_TEST_BUTTON:
                  armUp();while(infraredReceiverDecode.buttonState());
                  armStop();break;
            case IR_RETURN_BUTTON:
                  armDown();while(infraredReceiverDecode.buttonState());
                  armStop();break;
            default:break;
        }
    }
}

void runForward()
{
    Serial.println("run forward");
    baseShieldMotorDriver.runMotors(leftSpeed,-rightSpeed);
}

void runBack()
{
    Serial.println("run back");
    baseShieldMotorDriver.runMotors(-leftSpeed,rightSpeed);
}

void runLeft()
{
    Serial.println("run left");
    baseShieldMotorDriver.runMotors(-turnSpeed,-turnSpeed);
}

void runRight()
{
    Serial.println("run right");
    baseShieldMotorDriver.runMotors(turnSpeed,turnSpeed);
}

void runStop()
{
    Serial.println("run stop");
    baseShieldMotorDriver.stopMotors();
}

void armUp()
{
    Serial.println("arm up");
    motorDriver.run(-armSpeed);
}

void armDown()
{
    Serial.println("arm down");
    motorDriver.run(armSpeed);
}

void armStop()
{
    Serial.println("arm down");
    motorDriver.stop();
}
