/******************************************************************************
* File Name          : Me_LegoMotor.cpp
* Author             : Evan
* Updated            : Evan
* Version            : V1.0.0
* Date               : 4/18/2013
* Description        : Class for Makeblock Electronic modules of Me -  Lego 
                       Digital Bridge. The module is used to connect the NXT 
                       Lego Motor Driver to Arduino. The module can only be 
                       connected to the port 1, 2 of Me - Base Shield. One 
                       module can drive two Lego Motor.
* License            : CC-BY-SA 3.0
* Copyright (C) 2011 Hulu Robot Technology Co., Ltd. All right reserved.
*******************************************************************************/

#include "Me_LegoMotor.h"
//default port is PORT_1
Me_LegoMotor::Me_LegoMotor():softI2cMaster(mePort[PORT_1].outsidePin, mePort[PORT_1].innersidePin)
{
	port = PORT_1;
}

Me_LegoMotor::Me_LegoMotor(int portNum):softI2cMaster(mePort[portNum].outsidePin, mePort[portNum].innersidePin)
{
	port = portNum;
}
//ʼ
void Me_LegoMotor::begin()
{
	Me_BaseShield::begin();
	pinMode(mePort[port].innersidePin, OUTPUT);
	pinMode(mePort[port].outsidePin, OUTPUT);
	digitalWrite(mePort[port].innersidePin, HIGH);
	digitalWrite(mePort[port].outsidePin, HIGH);
}
//дһֽ
void Me_LegoMotor::writeByte(uint8_t Address, uint8_t dat)
{
	softI2cMaster.start(Address);
	softI2cMaster.write(dat);
	softI2cMaster.stop();
}
//ָλöһֽ
uint8_t Me_LegoMotor::readByte(uint8_t slAddress, uint8_t dtAddress)
{
	// issue a start condition, send device address and write direction bit
  if (!softI2cMaster.start(slAddress | I2C_WRITE)) return -1;

  // send the getting motor speed address
  if (!softI2cMaster.write(dtAddress)) return -1;

  // issue a repeated start condition, send device address and read direction bit
  if (!softI2cMaster.restart(slAddress | I2C_READ)) return -1;
  
 // int count = 1;
  // read data from the ultrasonic module
  unsigned char mtSpeed = softI2cMaster.read(1);

  // issue a stop condition
  softI2cMaster.stop();
  return mtSpeed;
}
//ָλöһĸֽ
uint32_t Me_LegoMotor::readLong(uint8_t slAddress,uint8_t dtAddress)
{
	uint8_t buf[4];
	buf[0] = readByte(slAddress,dtAddress);
	buf[1] = readByte(slAddress,dtAddress+1);
	buf[2] = readByte(slAddress,dtAddress+2);
	buf[3] = readByte(slAddress,dtAddress+3);
	return  buf[0] | (buf[1] << 8) | (buf[2] << 16) | (buf[3] << 24);
}
/*
//д֡ͷ
void Me_LegoMotor::writeHeader(uint8_t data1, uint8_t data2, uint8_t data3)
{
	writeByte(SLAVE_ADDRESS, data1);
	writeByte(SLAVE_ADDRESS, data2);
	writeByte(SLAVE_ADDRESS, data3);
}
//д
void Me_LegoMotor::writeControlCommand(uint8_t command)
{
	writeByte(command);
}
//д֡β
void Me_LegoMotor::writeEnd(uint8_t data)
{
	writeByte(data);
}
void writeCommand(uint8_t cmd)
{
	writeByte(SLAVE_ADDRESS, Data_Header_1);
	writeByte(SLAVE_ADDRESS, Data_Header_2);
	writeByte(SLAVE_ADDRESS, Data_Header_3);
	writeByte(SLAVE_ADDRESS, cmd);
}*/

/*Get status from the motors*/
/*Get speed status*/
uint8_t Me_LegoMotor::Motor1GetSpeedStatus()
{
  uint8_t speed;
  //writeCommand(Read_Command);
  writeByte(SLAVE_ADDRESS, Data_Header_1);
	writeByte(SLAVE_ADDRESS, Data_Header_2);
	writeByte(SLAVE_ADDRESS, Data_Header_3);
	writeByte(SLAVE_ADDRESS, Read_Command);
  speed = readByte(SLAVE_ADDRESS, Speed_RPM_M1);
  return speed;
}	
uint8_t Me_LegoMotor::Motor2GetSpeedStatus()
{
  uint8_t speed;
  //writeCommand(Read_Command);
  writeByte(SLAVE_ADDRESS, Data_Header_1);
	writeByte(SLAVE_ADDRESS, Data_Header_2);
	writeByte(SLAVE_ADDRESS, Data_Header_3);
	writeByte(SLAVE_ADDRESS, Read_Command);
  speed = readByte(SLAVE_ADDRESS, Speed_RPM_M2);
  return speed;
}	
/*Get seconds stasus*/
uint32_t Me_LegoMotor::Motor1GetSecondsStatus()
{
	uint32_t seconds;
	//writeCommand(Read_Command);
	writeByte(SLAVE_ADDRESS, Data_Header_1);
	writeByte(SLAVE_ADDRESS, Data_Header_2);
	writeByte(SLAVE_ADDRESS, Data_Header_3);
	writeByte(SLAVE_ADDRESS, Read_Command);
	seconds = readLong(SLAVE_ADDRESS,Duration_Seconds_M1);
	return seconds;
}
uint32_t Me_LegoMotor::Motor2GetSecondsStatus()
{
	uint32_t seconds;
	//writeCommand(Read_Command);
	writeByte(SLAVE_ADDRESS, Data_Header_1);
	writeByte(SLAVE_ADDRESS, Data_Header_2);
	writeByte(SLAVE_ADDRESS, Data_Header_3);
	writeByte(SLAVE_ADDRESS, Read_Command);
	seconds = readLong(SLAVE_ADDRESS,Duration_Seconds_M2);
	return seconds;
}
/*Get degrees status*/
uint32_t Me_LegoMotor::Motor1GetDegreesStatus()
{
	uint32_t degrees;
	//writeCommand(Read_Command);
	writeByte(SLAVE_ADDRESS, Data_Header_1);
	writeByte(SLAVE_ADDRESS, Data_Header_2);
	writeByte(SLAVE_ADDRESS, Data_Header_3);
	writeByte(SLAVE_ADDRESS, Read_Command);
	degrees = readLong(SLAVE_ADDRESS,Duration_Degrees_M1);
	return degrees;
}
uint32_t Me_LegoMotor::Motor2GetDegreesStatus()
{
	uint32_t degrees;
	//writeCommand(Read_Command);
	writeByte(SLAVE_ADDRESS, Data_Header_1);
	writeByte(SLAVE_ADDRESS, Data_Header_2);
	writeByte(SLAVE_ADDRESS, Data_Header_3);
	writeByte(SLAVE_ADDRESS, Read_Command);
	degrees = readLong(SLAVE_ADDRESS,Duration_Degrees_M2);
	return degrees;
}


//------------------------------------------------------------------------------
// ٶ
void Me_LegoMotor::MotorSetSpeed(uint8_t DataAddress,uint8_t direction, uint8_t speed)
{
	uint8_t databuffer[7];
	uint8_t i;
	databuffer[0] = DataAddress;
	databuffer[1] = 2;
	databuffer[2] = direction;
	databuffer[3] = speed;
	databuffer[4] = databuffer[0] + databuffer[1] + databuffer[2] + databuffer[3];
	databuffer[5] = databuffer[0] ^ databuffer[1] ^ databuffer[2] ^ databuffer[3];
	databuffer[6] = Data_End;
  //writeCommand(Write_Command);
  writeByte(SLAVE_ADDRESS, Data_Header_1);
	writeByte(SLAVE_ADDRESS, Data_Header_2);
	writeByte(SLAVE_ADDRESS, Data_Header_3);
	writeByte(SLAVE_ADDRESS, Write_Command);
	for(i=0;i<7;i++)	
	{ 	     		
	    writeByte(SLAVE_ADDRESS, databuffer[i]);    	
	}
}
//ʱ
void Me_LegoMotor::MotorSetDuration(uint8_t DataAddress, uint32_t duration)
{
	uint8_t databuffer[9];
	uint8_t i;
	databuffer[0] = DataAddress;
	databuffer[1] = 4;
	databuffer[2] = duration & 0xFF;
  databuffer[3] = (duration >>  8) & 0xFF;
  databuffer[4] = (duration >> 16) & 0xFF;
  databuffer[5] = (duration >> 24) & 0xFF;
  databuffer[6] = databuffer[0] + databuffer[1] + databuffer[2] + databuffer[3] + databuffer[4] + databuffer[5];
	databuffer[7] = databuffer[0] ^ databuffer[1] ^ databuffer[2] ^ databuffer[3] ^ databuffer[4] ^ databuffer[5];
	databuffer[8] = Data_End;
	//writeCommand(Write_Command);
  writeByte(SLAVE_ADDRESS, Data_Header_1);
	writeByte(SLAVE_ADDRESS, Data_Header_2);
	writeByte(SLAVE_ADDRESS, Data_Header_3);
	writeByte(SLAVE_ADDRESS, Write_Command);
	for(i=0;i<9;i++)	
	{ 	     		
	    writeByte(SLAVE_ADDRESS, databuffer[i]);    	
	}
}
//ͬʱٶȺʱ
void Me_LegoMotor::MotorSetSpeedAndDuration(
	uint8_t DataAddress_SP,uint8_t direction, uint8_t speed,
	uint8_t DataAddress_DU,uint32_t duration)
{
	MotorSetSpeed(DataAddress_SP, direction, speed);
	MotorSetDuration(DataAddress_DU, duration);
}

/*Motor run unlimited*/
//1Թģʽת
void Me_LegoMotor::Motor1RunUnlimited_Power(uint8_t direction, uint8_t speed)	//in range [-100, +100]
{
	MotorSetSpeedAndDuration(Speed_Power_M1,direction,speed,Duration_Unlimited_M1,0);
}
//2Թģʽת
void Me_LegoMotor::Motor2RunUnlimited_Power(uint8_t direction, uint8_t speed)	//in range [-100, +100]
{
	MotorSetSpeedAndDuration(Speed_Power_M2,direction,speed,Duration_Unlimited_M2,0);
}
//1ģʽת
void Me_LegoMotor::Motor1RunUnlimited_RPM(uint8_t direction, uint8_t speed)	//in range [-160rpm, +160rpm]
{
	MotorSetSpeedAndDuration(Speed_RPM_M1,direction,speed,Duration_Unlimited_M1,0);
}
//2ģʽת
void Me_LegoMotor::Motor2RunUnlimited_RPM(uint8_t direction, uint8_t speed)	//in range [-160rpm, +160rpm]
{
	MotorSetSpeedAndDuration(Speed_RPM_M2,direction,speed,Duration_Unlimited_M2,0);
}


/*Runs the motors for a given number of seconds*/
void Me_LegoMotor::Motor1RunSeconds_Power(uint8_t direction, uint8_t speed,uint32_t seconds,Completion_Wait wait_for_completion)
{
	uint8_t s;
	MotorSetSpeedAndDuration(Speed_Power_M1,direction,speed,Duration_Seconds_M1,seconds);
	if(wait_for_completion == Completion_Wait_For)
	{
		//delay(50);
		s = Motor1GetSecondsStatus();
	  while(s != 0)
	  {
	  	//delay(50);
	  	s = Motor1GetSecondsStatus();
	  }
	}	
}
void Me_LegoMotor::Motor2RunSeconds_Power(uint8_t direction, uint8_t speed,uint32_t seconds,Completion_Wait wait_for_completion)
{
	uint8_t s;
	MotorSetSpeedAndDuration(Speed_Power_M2,direction,speed,Duration_Seconds_M2,seconds);
	if(wait_for_completion == Completion_Wait_For)
	{
		//delay(50);
		s = Motor2GetSecondsStatus();
	  while(s != 0)
	  {
	  	//delay(50);
	  	s = Motor2GetSecondsStatus();
	  }
	}	
}
void Me_LegoMotor::Motor1RunSeconds_RPM(uint8_t direction, uint8_t speed,uint32_t seconds,Completion_Wait wait_for_completion)
{
	uint8_t s;
	MotorSetSpeedAndDuration(Speed_RPM_M1,direction,speed,Duration_Seconds_M1,seconds);
	if(wait_for_completion == Completion_Wait_For)
	{
		//delay(50);
		s = Motor1GetSecondsStatus();
	  while(s != 0)
	  {
	  	//delay(50);
	  	s = Motor1GetSecondsStatus();
	  }
	}	
}
void Me_LegoMotor::Motor2RunSeconds_RPM(uint8_t direction, uint8_t speed,uint32_t seconds,Completion_Wait wait_for_completion)
{
	uint8_t s;
	MotorSetSpeedAndDuration(Speed_RPM_M2,direction,speed,Duration_Seconds_M2,seconds);
	if(wait_for_completion == Completion_Wait_For)
	{
		//delay(50);
		s = Motor2GetSecondsStatus();
	  while(s != 0)
	  {
	  	//delay(50);
	  	s = Motor2GetSecondsStatus();
	  }
	}	
}

/*Turns the motors the specified number of degrees*/
void Me_LegoMotor::Motor1RunDegrees_Power(uint8_t direction, uint8_t speed,uint32_t degrees,Completion_Wait wait_for_completion)
{
	uint8_t s;
	MotorSetSpeedAndDuration(Speed_Power_M1,direction,speed,Duration_Degrees_M1,degrees);
	if(wait_for_completion == Completion_Wait_For)
	{
		//delay(50);
		s = Motor1GetDegreesStatus();
	  while(s != 0)
	  {
	  	//delay(50);
	  	s = Motor1GetDegreesStatus();
	  }
	}	
}
void Me_LegoMotor::Motor2RunDegrees_Power(uint8_t direction, uint8_t speed,uint32_t degrees,Completion_Wait wait_for_completion)
{
	uint8_t s;
	MotorSetSpeedAndDuration(Speed_Power_M2,direction,speed,Duration_Degrees_M2,degrees);
	if(wait_for_completion == Completion_Wait_For)
	{
		//delay(50);
		s = Motor2GetDegreesStatus();
	  while(s != 0)
	  {
	  	//delay(50);
	  	s = Motor2GetDegreesStatus();
	  }
	}	
}
void Me_LegoMotor::Motor1RunDegrees_RPM(uint8_t direction, uint8_t speed,uint32_t degrees,Completion_Wait wait_for_completion)
{
	uint8_t s;
	MotorSetSpeedAndDuration(Speed_RPM_M1,direction,speed,Duration_Degrees_M1,degrees);
	if(wait_for_completion == Completion_Wait_For)
	{
		//delay(50);
		s = Motor1GetDegreesStatus();
	  while(s != 0)
	  {
	  	//delay(50);
	  	s = Motor1GetDegreesStatus();
	  }
	}	
}
void Me_LegoMotor::Motor2RunDegrees_RPM(uint8_t direction, uint8_t speed,uint32_t degrees,Completion_Wait wait_for_completion)
{
	uint8_t s;
	MotorSetSpeedAndDuration(Speed_RPM_M2,direction,speed,Duration_Degrees_M2,degrees);
	if(wait_for_completion == Completion_Wait_For)
	{
		//delay(50);
		s = Motor2GetDegreesStatus();
	  while(s != 0)
	  {
	  	//delay(50);
	  	s = Motor2GetDegreesStatus();
	  }
	}	
}

/*runs the motors the specified number of rotations*/
void Me_LegoMotor::Motor1RunRotations_Power(uint8_t direction, uint8_t speed,uint32_t rotations,Completion_Wait wait_for_completion)
{
	Motor1RunDegrees_Power(direction,speed,360 * rotations,wait_for_completion);
}
void Me_LegoMotor::Motor2RunRotations_Power(uint8_t direction, uint8_t speed,uint32_t rotations,Completion_Wait wait_for_completion)
{
	Motor2RunDegrees_Power(direction,speed,360 * rotations,wait_for_completion);
}
void Me_LegoMotor::Motor1RunRotations_RPM(uint8_t direction, uint8_t speed,uint32_t rotations,Completion_Wait wait_for_completion)
{
	Motor1RunDegrees_RPM(direction,speed,360 * rotations,wait_for_completion);
}
void Me_LegoMotor::Motor2RunRotations_RPM(uint8_t direction, uint8_t speed,uint32_t rotations,Completion_Wait wait_for_completion)
{
	Motor2RunDegrees_RPM(direction,speed,360 * rotations,wait_for_completion);
}

/*Stop the motors*/
void Me_LegoMotor::Motor1Stop_Brake()
{
	Motor1RunUnlimited_RPM(0,0);
}
void Me_LegoMotor::Motor2Stop_Brake()
{
	Motor2RunUnlimited_RPM(0,0);
}

void Me_LegoMotor::Motor1Stop_Coast()
{
	Motor1RunUnlimited_Power(0,0);
}
void Me_LegoMotor::Motor2Stop_Coast()
{
	Motor2RunUnlimited_Power(0,0);
}