/******************************************************************************
* File Name          : Me_LegoMotor.h
* Author             : Evan
* Updated            : Evan
* Version            : V1.0.0
* Date               : 4/18/2013
* Description        : Class for Makeblock Electronic modules of Me -  Lego 
                       Digital Bridge. The module is used to connect the NXT 
                       Lego Motor Driver to Arduino. The module can only be 
                       connected to the port 1, 2 of Me - Base Shield. One 
                       module can drive two Lego Motor.
* License            : CC-BY-SA 3.0
* Copyright (C) 2011 Hulu Robot Technology Co., Ltd. All right reserved.
*******************************************************************************/

#include <Arduino.h>
#include <Me_BaseShield.h>
#include <Me_I2cMaster.h>


#ifndef Me_LegoMotor_h
#define Me_LegoMotor_h
#define SLAVE_ADDRESS 0x08
#define Data_Header_1 0x55
#define Data_Header_2 0xAA
#define Data_Header_3 0x7E
#define Data_End 0x0D
//Control command
#define Read_Command 0x01
#define Write_Command 0x02
//Speed data encode
#define Speed_Power_M1 0x01   
#define Speed_RPM_M1 0x02
#define Speed_Power_M2 0x81
#define Speed_RPM_M2 0x82 
#define Forward 0x01
#define Backward 0x00
//Duration data encode
#define Duration_Unlimited_M1 0x21
#define Duration_Seconds_M1 0x22
#define Duration_Degrees_M1 0x27
#define Duration_Rotations_M1 0x27
#define Duration_Unlimited_M2 0x41
#define Duration_Seconds_M2 0x42
#define Duration_Degrees_M2 0x47
#define Duration_Rotations_M2 0x47

typedef enum {
   Completion_Dont_Wait    = 0x00,  
 	 Completion_Wait_For     = 0x01  
 } Completion_Wait;

class Me_LegoMotor
{
public:
	Me_LegoMotor();
	//portNum can only be 1 & 2
	Me_LegoMotor(int portNum);
	void begin();
	void writeByte(uint8_t Address, uint8_t dat);
	uint8_t readByte(uint8_t slAddress, uint8_t dtAddress);
	uint32_t readLong(uint8_t slAddress,uint8_t dtAddress);
	//void writeCommand(uint8_t cmd);
	uint8_t Motor1GetSpeedStatus();
	uint8_t Motor2GetSpeedStatus();
	uint32_t Motor1GetSecondsStatus();
	uint32_t Motor2GetSecondsStatus();
	uint32_t Motor1GetDegreesStatus();
	uint32_t Motor2GetDegreesStatus();
	//void SendData(uint8_t dat[10]);
	void MotorSetSpeed(uint8_t DataAddress, uint8_t direction, uint8_t speed);
	void MotorSetDuration(uint8_t DataAddress, uint32_t duration);
	void MotorSetSpeedAndDuration(uint8_t DataAddress_SP, uint8_t direction, uint8_t speed, uint8_t DataAddress_DU, uint32_t duration);
	void Motor1RunUnlimited_Power(uint8_t direction, uint8_t speed);
	void Motor2RunUnlimited_Power(uint8_t direction, uint8_t speed);
	void Motor1RunUnlimited_RPM(uint8_t direction, uint8_t speed);
	void Motor2RunUnlimited_RPM(uint8_t direction, uint8_t speed);
	void Motor1RunSeconds_Power(uint8_t direction, uint8_t speed, uint32_t seconds, Completion_Wait wait_for_completion);
	void Motor2RunSeconds_Power(uint8_t direction, uint8_t speed, uint32_t seconds, Completion_Wait wait_for_completion);
	void Motor1RunSeconds_RPM(uint8_t direction, uint8_t speed, uint32_t seconds, Completion_Wait wait_for_completion);
	void Motor2RunSeconds_RPM(uint8_t direction, uint8_t speed, uint32_t seconds, Completion_Wait wait_for_completion);
	void Motor1RunDegrees_Power(uint8_t direction, uint8_t speed, uint32_t degrees, Completion_Wait wait_for_completion);
	void Motor2RunDegrees_Power(uint8_t direction, uint8_t speed, uint32_t degrees, Completion_Wait wait_for_completion);
	void Motor1RunDegrees_RPM(uint8_t direction, uint8_t speed, uint32_t degrees, Completion_Wait wait_for_completion);
	void Motor2RunDegrees_RPM(uint8_t direction, uint8_t speed, uint32_t degrees, Completion_Wait wait_for_completion);
	void Motor1RunRotations_Power(uint8_t direction, uint8_t speed, uint32_t rotations, Completion_Wait wait_for_completion);
	void Motor2RunRotations_Power(uint8_t direction, uint8_t speed, uint32_t rotations, Completion_Wait wait_for_completion);
	void Motor1RunRotations_RPM(uint8_t direction, uint8_t speed, uint32_t rotations, Completion_Wait wait_for_completion);
	void Motor2RunRotations_RPM(uint8_t direction, uint8_t speed, uint32_t rotations, Completion_Wait wait_for_completion);
	void Motor1Stop_Brake();
	void Motor2Stop_Brake();
	void Motor1Stop_Coast();
	void Motor2Stop_Coast();
private:
	uint8_t Databuf[12]; 
	int port;
	Me_SoftI2cMaster softI2cMaster;
};
#endif